/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * bma2x2_hal.c
 *
 *  Created on: Sep 1, 2015
 *      Author: alejandro
 ****************************************************************************************
 */
/*---------------------------------------------------------------------------*/
/* Includes*/
/*---------------------------------------------------------------------------*/
#include "tx_api.h"
#include "r_sci_spi.h"
#include "bma2x2.h"
#include "bma2x2_ext.h"
#include "acc.h"
#include "util.h"
#include "config.h"
#include "acc_thread.h"


#if ACC_RATE == 32
#   define BMA2x2_BW_CONFIG BMA2x2_BW_31_25HZ
#elif ACC_RATE == 62
#   define BMA2x2_BW_CONFIG BMA2x2_BW_62_50HZ
#elif ACC_RATE == 125
#   define BMA2x2_BW_CONFIG BMA2x2_BW_125HZ
#elif ACC_RATE == 250
#   define BMA2x2_BW_CONFIG BMA2x2_BW_250HZ
#elif ACC_RATE == 500
#   define BMA2x2_BW_CONFIG BMA2x2_BW_500HZ
#else
#   error Select a proper low pass filter setting for accelerometer
#endif

#if ACC_G_RANGE == 2
#   define BMA2x2_RANGE_CONFIG  BMA2x2_RANGE_2G
#elif ACC_G_RANGE == 4
#   define BMA2x2_RANGE_CONFIG  BMA2x2_RANGE_4G
#elif ACC_G_RANGE == 8
#   define BMA2x2_RANGE_CONFIG  BMA2x2_RANGE_8G
#else
#   error Select a proper G range
#endif

/*----------------------------------------------------------------------------*
*   The following functions are used for reading and writing of
*   sensor data using I2C or SPI communication
*----------------------------------------------------------------------------*/
/*  \Brief: The function is used as SPI bus write
 *  \Return : Status of the SPI write
 *  \param dev_addr : The device address of the sensor
 *  \param reg_addr : Address of the first register, will data is going to be written
 *  \param reg_data : It is a value hold in the array,
 *      will be used for write the value into the register
 *  \param cnt : The no of byte of data to be write
 */
static s8 BMA2x2_SPI_bus_write(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt);
/*  \Brief: The function is used as SPI bus read
 *  \Return : Status of the SPI read
 *  \param dev_addr : The device address of the sensor
 *  \param reg_addr : Address of the first register, will data is going to be read
 *  \param reg_data : This data read from the sensor, which is hold in an array
 *  \param cnt : The no of byte of data to be read */
static s8 BMA2x2_SPI_bus_read(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt);

/*  \Brief: The function is used as SPI bus read
 *  \Return : Status of the SPI read
 *  \param dev_addr : The device address of the sensor
 *  \param reg_addr : Address of the first register, will data is going to be read
 *  \param reg_data : This data read from the sensor, which is hold in an array
 *  \param cnt : The no of byte of data to be read */
static s8 BMA2x2_SPI_bus_bread(u8 dev_addr, u8 reg_addr, u8 *reg_data, u32 cnt);

/********************End of SPI function declarations***********************/
/*  Brief : The delay routine
 *  \param : delay in ms
 */
static void BMA2x2_delay_msek(u32 msek);
/*----------------------------------------------------------------------------*
*  struct bma2x2_t parameters can be accessed by using bma2x2
 *  bma2x2_t having the following parameters
 *  Bus write function pointer: BMA2x2_WR_FUNC_PTR
 *  Bus read function pointer: BMA2x2_RD_FUNC_PTR
 *  Burst read function pointer: BMA2x2_BRD_FUNC_PTR
 *  Delay function pointer: delay_msec
 *  I2C address: dev_addr
 *  Chip id of the sensor: chip_id
 *---------------------------------------------------------------------------*/
static struct bma2x2_t bma2x2 =
{
    .bus_write = BMA2x2_SPI_bus_write,
    .bus_read = BMA2x2_SPI_bus_read,
    .burst_read = BMA2x2_SPI_bus_bread,
    .delay_msec = BMA2x2_delay_msek,
};

static ssp_err_t bma2x2_start (void);
static void bma2x2_deselect ();
static void bma2x2_select ();

/************** SPI buffer length ******/
#define SPI_BUFFER_LEN 5

/*----------------------------------------------------------------------------*
*  V_BMA2x2RESOLUTION_u8R used for selecting the accelerometer resolution
 *  12 bit
 *  14 bit
 *  10 bit
*----------------------------------------------------------------------------*/
//extern u8 V_BMA2x2RESOLUTION_U8;

typedef struct
{
    uint16_t new : 1;
    uint16_t reserved : 3;
    uint16_t value : 12;
}bma2x2_axis_t;

typedef struct
{
    bma2x2_axis_t x;
    bma2x2_axis_t y;
    bma2x2_axis_t z;
}bma2x2_frame_data_t;

typedef struct
{
    const spi_api_t *api;
    spi_instance_t *spi;
    spi_ctrl_t SPIx;
    bool running;
    struct bma2x2_accel_data xyz;
    struct bma2x2_t *bma2x2;
    const external_irq_instance_t *irq;
    bmx2x2_stat_t stat;
    uint8_t fifo_frame_count;
    bma2x2_frame_data_t fifo_a[BMA2X2_FIFO_DEPTH];
    bma2x2_frame_data_t fifo_b[BMA2X2_FIFO_DEPTH];
    bma2x2_frame_data_t *live; // live buffer
    bma2x2_frame_data_t *ready; // standby buffer
}bma2x2_context_t;

// abstracted view of the accelerometer
typedef struct
{
    acc_unit_t set_a[ACC_SET_COUNT];
    acc_unit_t set_b[ACC_SET_COUNT];   // read to be read buffer
    int idx;
    acc_unit_t *live;   // points to the live data set .. never access this directly
    acc_unit_t *ready;
    TX_MUTEX set_mutex;
//mfr    TX_SEMAPHORE set_ready;
}hal_acc_context_t;

TX_SEMAPHORE *acc_set_ready;

static bma2x2_context_t context =
{
    .api = &g_spi_on_sci,
    .spi = (spi_instance_t *)&g_spi_bma2x2,
    .xyz = {0, 0, 0}, // some default ZEROs
    .running = false,
    .bma2x2 = &bma2x2,
    .irq = &g_accel_irq,
};

// hardware context
//static bma2x2_context_t *bma2x2_context = &context;

// abstracted context of the accelerometer
static hal_acc_context_t hal_acc_context;

ssp_err_t acc_start (void)
{
    ssp_err_t err = SSP_SUCCESS;
    bma2x2_context_t *c = &context;
    hal_acc_context_t *ac = &hal_acc_context;

    // discard const qualifier
    spi_cfg_t cfg = *(spi_cfg_t *)c->spi->p_cfg;
    c->spi->p_cfg = &cfg;
    c->live = c->fifo_a;
    c->ready = c->fifo_b;
    // abstracted context init
    ac->live = ac->set_a;
    ac->ready = ac->set_b;
    ac->idx = 0;
    // abstracted mutexes and semaphores
    tx_mutex_create(&ac->set_mutex, "ACC set mutex", 0);
    // start the hardware
    err = bma2x2_start ();
    // ok?
    if (SSP_SUCCESS == err)
    {
        // at this point we are ready
        c->running = true;
    }

    return err;
}

ssp_err_t acc_poll (void);
ssp_err_t acc_poll (void)
{
    ssp_err_t err = SSP_SUCCESS;
    struct bma2x2_accel_data accel;;

    bma2x2_read_accel_xyz(&accel);

    context.xyz.x = accel.x;
    context.xyz.y = accel.y;
    context.xyz.z = accel.z;

    return err;
}

ssp_err_t acc_get_last (acc_unit_t *o);
ssp_err_t acc_get_last (acc_unit_t *o)
{
    ssp_err_t err = SSP_SUCCESS;

    o->x = context.xyz.x;
    o->y = context.xyz.y;
    o->z = context.xyz.z;

    return err;
}

#define FIFO_MODE_FIFO  0x01

static ssp_err_t bma2x2_start (void)
{
    ssp_err_t  err = SSP_SUCCESS;
    bma2x2_context_t *c = &context;

    if (c->running)
    {
        // NOTE:
        // maybe need an SSP_ERR_ALREADY_RUNNING
        err = SSP_ERR_ALREADY_OPEN;
    }
    else
    {
        bma2x2_deselect ();

        /* Clear SPI device context */
        memset(&c->SPIx, 0, sizeof(c->SPIx));
        // sanity check
        if (c->spi->p_cfg)
        {
            spi_cfg_t *SPIx_cfg = (spi_cfg_t *)c->spi->p_cfg;
            /* Open logical SPI */
             err = c->api->open (&c->SPIx, SPIx_cfg);
        }
    }

    if (SSP_SUCCESS == err)
    {
        // BE 40 R (3E 40)
        // 91 00 R (11 00)
        bma2x2_init (&bma2x2);
        /* Set the power mode as NORMAL*/
        // 92 00 R (12 00)
        // 12 00 W (12 00)
        // 11 80 W (11 80)
        // 3E 40 W (3E 40)
        // 91 00 R (11 80)
        // 11 00 W (11 00)
        bma2x2_set_power_mode(BMA2x2_MODE_NORMAL);
        // setup desired low pass filter ...
        bma2x2_set_bw(BMA2x2_BW_CONFIG);
        // setup the desired G range +-2, +-4 ...
        bma2x2_set_range (BMA2x2_RANGE_CONFIG);
        // A0 07 R (20 05) // (INT_OUT_CTRL) Contains the behavioural configuration (electrical behavior) of the interrupt pins.
        // 20 05 R (20 05) // (INT_OUT_CTRL) INT1 PUSH PULL, LVL HIGH
        // push pull
        bma2x2_set_intr_output_type (BMA2x2_INTR1_OUTPUT, PUSS_PULL);
        // INT1 active high
        // A0 05 R (20 05) // (INT_OUT_CTRL)
        // 20 05 R (20 05) // (INT_OUT_CTRL) INT1 PUSH PULL, LVL HIGH
        bma2x2_set_intr_level (BMA2x2_INTR1_LEVEL, ACTIVE_HIGH);
        // FIFO mode FIFO
        // BE 40 R (3E 40) // FIFO_CONFIG_1
        // 3E 40 W (3E 40) //
        // BE 40 R (3E 40)
        bma2x2_set_fifo_mode (FIFO_MODE_FIFO);
        // now configure for FIFO full interrupt to INT1
        // 9A 04 R (1A 04) INT_MAP_1
        // 1A 04 W (1A 04) INT1 FIFO FULL
        bma2x2_set_intr1_fifo_full (INTR_ENABLE);
        // enable FIFO interrupt
        bma2x2_set_intr_fifo_full(INTR_ENABLE);
        // initialize the NVIC
        c->irq->p_api->open (c->irq->p_ctrl, c->irq->p_cfg);
    }

    return err;
}

/*  \Brief: The function is used as SPI bus write
 *  \Return : Status of the SPI write
 *  \param dev_addr : The device address of the sensor
 *  \param reg_addr : Address of the first register, will data is going to be written
 *  \param reg_data : It is a value hold in the array,
 *      will be used for write the value into the register
 *  \param cnt : The no of byte of data to be write
 */
static s8 BMA2x2_SPI_bus_write(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt)
{
    s32 iError = 0;
    u8 array[SPI_BUFFER_LEN * 2];
    u8 dummy[SPI_BUFFER_LEN * 2];
    u8 stringpos = 0;
    bma2x2_context_t *c = &context;
    (void) ((dev_addr));

    for (stringpos = 0; stringpos < cnt; stringpos++)
    {
        /* the operation of (reg_addr++)&0x7F done: because it ensure the
           0 and 1 of the given value
           It is done only for 8bit operation*/
        array[stringpos * 2] = (reg_addr++) & 0x7F;
        array[stringpos * 2 + 1] = *(reg_data + stringpos);
    }
    // select
    bma2x2_select ();
    // do the read
    ssp_err_t err = (ssp_err_t)c->api->writeRead(&context.SPIx, array, dummy, (uint32_t)(cnt*2), SPI_BIT_WIDTH_8_BITS);
    tx_semaphore_get(&g_acc_semaphore, TX_WAIT_FOREVER);
    // deselect
    bma2x2_deselect ();
    // return code
    iError = err;
    /* Please take the below function as your reference
     * for write the data using SPI communication
     * add your SPI write function here.
     * "IERROR = SPI_WRITE_STRING(ARRAY, CNT*2)"
     * iError is an return value of SPI write function
     * Please select your valid return value
     * In the driver SUCCESS defined as 0
     * and FAILURE defined as -1
     */
    return (s8)iError;
}

/*  \Brief: The function is used as SPI bus read
 *  \Return : Status of the SPI read
 *  \param dev_addr : The device address of the sensor
 *  \param reg_addr : Address of the first register, will data is going to be read
 *  \param reg_data : This data read from the sensor, which is hold in an array
 *  \param cnt : The no of byte of data to be read */
static s8 BMA2x2_SPI_bus_read(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt)
{
    s32 iError=0;
    u8 array[SPI_BUFFER_LEN]={0xFF};
    u8 stringpos;
    bma2x2_context_t *c = &context;
    (void) ((dev_addr));
    /*  For the SPI mode only 7 bits of register addresses are used.
    The MSB of register address is declared the bit what functionality it is
    read/write (read as 1/write as 0)*/
    array[0] = reg_addr | 0x80;/*read routine is initiated register address is mask with 0x80*/
    // select
    bma2x2_select ();
    // do the write/read
    ssp_err_t err = c->api->writeRead(&context.SPIx, array, array, (uint32_t)(cnt + 1), SPI_BIT_WIDTH_8_BITS);
    tx_semaphore_get(&g_acc_semaphore, TX_WAIT_FOREVER);
    // deselect
    bma2x2_deselect ();
    // return code
    iError = err;
    /*
    * Please take the below function as your reference for
    * read the data using SPI communication
    * " IERROR = SPI_READ_WRITE_STRING(ARRAY, ARRAY, CNT+1)"
    * add your SPI read function here
    * iError is an return value of SPI read function
    * Please select your valid return value
    * In the driver SUCCESS defined as 0
    * and FAILURE defined as -1
    * Note :
    * This is a full duplex operation,
    * The first read data is discarded, for that extra write operation
    * have to be initiated. For that cnt+1 operation done in the SPI read
    * and write string function
    * For more information please refer data sheet SPI communication:
    */
    for (stringpos = 0; stringpos < cnt; stringpos++)
    {
        *(reg_data + stringpos) = array[stringpos+1];
    }

    return (s8)iError;
}

/*  \Brief: The function is used as SPI bus burst read
 *  \Return : Status of the SPI read
 *  \param dev_addr : The device address of the sensor
 *  \param reg_addr_start : Address of the first register, will data is going to be read
 *  \param reg_data : This data read from the sensor, which is hold in an array
 *  \param cnt : The no of byte of data to be read */
static s8 BMA2x2_SPI_bus_bread(u8 dev_addr, u8 reg_addr, u8 *reg_data, u32 cnt)
{
    s32 iError=0;
    u8 reg[1]={0xFF};
    bma2x2_context_t *c = &context;
    (void) ((dev_addr));
    /*  For the SPI mode only 7 bits of register addresses are used.
    The MSB of register address is declared the bit what functionality it is
    read/write (read as 1/write as 0)*/
    reg[0] = reg_addr | 0x80;/*read routine is initiated register address is mask with 0x80*/
    // select
    bma2x2_select ();
    // do the write/read
    ssp_err_t err = c->api->write (&context.SPIx, reg, sizeof(reg), SPI_BIT_WIDTH_8_BITS);
    tx_semaphore_get(&g_acc_semaphore, TX_WAIT_FOREVER);
    if (SSP_SUCCESS == err)
    {
        err = c->api->read(&context.SPIx, reg_data, cnt, SPI_BIT_WIDTH_8_BITS);
        tx_semaphore_get(&g_acc_semaphore, TX_WAIT_FOREVER);
    }

    // deselect
    bma2x2_deselect ();
    // return code
    iError = err;

    return (s8)iError;
}

#define OS_TICK_MS 10

/*  Brief : The delay routine
 *  \param : delay in ms
 */
static void BMA2x2_delay_msek(u32 msek)
{
    int rem = (int)(msek % OS_TICK_MS);
    int ticks = (int)(msek / OS_TICK_MS);

    if (rem > 0)
    {
        // round to the next tick
        ticks++;
    }
    // wait
    tx_thread_sleep((ULONG)ticks);
}

static void bma2x2_deselect (void)
{
    const ioport_instance_t *p = &g_ioport;

    p->p_api->pinWrite(ACC_CS_PIN, IOPORT_LEVEL_HIGH);
}

static void bma2x2_select (void)
{
    const ioport_instance_t *p = &g_ioport;

    p->p_api->pinWrite(ACC_CS_PIN, IOPORT_LEVEL_LOW);
}

// extend the sign if necessary
#define BMA2x2_DECODE(V) (int16_t)(V & (1 << 11) ? (V | 0xf800) : V)

static bool acc_abstract_service (bma2x2_context_t *hc, hal_acc_context_t *ac)
{
    bool data_ready = false;

    for (int i = 0; i < BMA2X2_FIFO_DEPTH; i++)
    {
        if (ac->idx < ACC_SET_COUNT)
        {
            // nothing special to be done
        }
        else
        {
            // we have filled the abstract set
            tx_mutex_get (&ac->set_mutex, TX_WAIT_FOREVER);
            acc_unit_t *temp;
            // ping pong
            temp = ac->live;
            ac->live = ac->ready;
            ac->ready = temp;
            // done
            tx_mutex_put (&ac->set_mutex);
            // data is now ready for further processing
            data_ready = true;
            // start again
            ac->idx = 0;
        }

        // copy the data
        ac->live[ac->idx].x = BMA2x2_DECODE(hc->ready[i].x.value);
        ac->live[ac->idx].y = BMA2x2_DECODE(hc->ready[i].y.value);
        ac->live[ac->idx].z = BMA2x2_DECODE(hc->ready[i].z.value);
        // next
        ac->idx++;
    }

    return data_ready;
}

static bool acc_fifo_full_service (bma2x2_context_t *c)
{
    void *temp;
    ssp_err_t err = SSP_SUCCESS;

    // reading the FIFO will clear the interrupt
    err = c->bma2x2->BMA2x2_BURST_READ_FUNC(
            c->bma2x2->dev_addr,
            BMA2x2_FIFO_DATA_OUTPUT_REG,
            (u8 *)c->live, BMA2X2_FIFO_SIZE);
    if (SSP_SUCCESS != err)
    {
        // ERROR:
        while(1) {;}
    }

    // ping pong the buffers (no protection required since there is no dirtect access to this
    // buffers from outside
    temp = c->live;
    c->live = c->ready;
    c->ready = temp;
    //
    return acc_abstract_service (c, &hal_acc_context);
}

bool acc_int_service (void)
{
    bool data_ready = false;
    bma2x2_context_t *c = &context;
    // NOTE:
    // read the source of the interrupt
    bma2x2_get_intr_stat((u8 *)&c->stat);
    bma2x2_get_fifo_frame_count ((u8 *)&c->fifo_frame_count);

    // is this the FIFO full interrupt?
    if (c->stat.bits.fifo_full_int)
    {
        data_ready = acc_fifo_full_service (c);
        //
        bma2x2_get_fifo_frame_count ((u8 *)&c->fifo_frame_count);
    }

    return data_ready;
}

void acc_set_get (acc_unit_t *out, int max)
{
    hal_acc_context_t *c = &hal_acc_context;

    // we have filled the abstract set
    tx_mutex_get (&c->set_mutex, TX_WAIT_FOREVER);
    //copy over the data from the standby set
    int total = MIN (ACC_SET_COUNT, max);

    for (int i = 0; i < total; i++)
    {
        // copy over the data
        out[i].x = c->ready[i].x;
        out[i].y = c->ready[i].y;
        out[i].z = c->ready[i].z;
    }
    // done
    tx_mutex_put (&c->set_mutex);
}

bool acc_started (void)
{
    return context.running;
}
