/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#include <stdbool.h>
#include <stdlib.h>
#include <math.h>
#include "humidity.h"
#include "sensor_thread.h"

// Humidity sensor timeout
#define HUMIDITY_I2C_TIMEOUT    100

// forward function declarations
static HumidityErrorCode humidity_reset(void);
static void humidity_delay_msek(uint16_t msek);
static float humidity_decode(uint16_t enc);
static HumidityErrorCode humidity_write(uint8_t dev_addr, uint8_t reg_addr, uint8_t *reg_data, uint8_t cnt);
static HumidityErrorCode humidity_read(uint8_t dev_addr, uint8_t *reg_data, uint8_t cnt);

/***************************************************************************/
/*			humidity_init - humidity.h for description			           */
/***************************************************************************/
HumidityErrorCode humidity_init(void)
{
    HumidityErrorCode error = HUMIDITY_OK;

    // reset device
    error = humidity_reset();
    
    return error;
}

/***************************************************************************/
/*			humidity_trigger - humidity.h for description		           */
/***************************************************************************/
HumidityErrorCode humidity_trigger(void)
{
    HumidityErrorCode error = HUMIDITY_OK;
    
    error = humidity_write(SHT2x_ADDRESS, SHT2x_READ_RH_NHM, NULL, 0);
    
    if (HUMIDITY_OK == error)
    {
        // detlay a few milliseconds
        humidity_delay_msek(10);
    }
    
    return error;
}

/***************************************************************************/
/*			humidity_read_humidity - humidity.h for description		       */
/***************************************************************************/
HumidityErrorCode humidity_read_humidity(float *humidity)
{
    HumidityErrorCode error = HUMIDITY_OK;
    uint16_t raw = 0;
    
    error = humidity_read(SHT2x_ADDRESS, (uint8_t *)&raw, 2);
    
    if (HUMIDITY_OK == error)
    {
        *humidity = humidity_decode(raw);
    }
    else
    {
        *humidity = NAN;
        // TODO: reset the I2C bus
    }
    
    return error;
}

/***************************************************************************/
/**
 *	@brief Reset humidity sensor
 *
 ***************************************************************************/
static HumidityErrorCode humidity_reset(void)
{
    HumidityErrorCode error = HUMIDITY_OK;
    
    error = humidity_write(SHT2x_ADDRESS, SHT2x_SOFT_RESET, NULL, 0);
    
    if (HUMIDITY_OK == error)
    {
        // detlay a few milliseconds
        humidity_delay_msek(10);
    }
    
    return error;
}

#define SHT25_IS_TEMP(val)  (0 == (val & 2))

static float humidity_decode(uint16_t enc)
{
    /* resolve the encoded temperature */
    static float out = 0.0f;
    
    if (SHT25_IS_TEMP (enc))
    {
        /* The temperature T is calculated by inserting temperature
        signal output ST into the following formula (result in 8C), no
        matter which resolution is chosen:
        T = 175.72 * (St/2^16) - 46.85 */
        float temp = enc;
        temp /= 65536;
        temp *= 175.72f;
        temp -= 46.85f;
        // only interested in reporting humidity reading
        // out = temp;
    }
    else
    {
        /* With the relative humidity signal output SRH the relative
        humidity RH is obtained by the following formula (result in
        %RH), no matter which resolution is chosen:
        RH = 125 * (Sh/2^16) - 6 */
        float humidity = enc;
        humidity /= 65536;
        humidity *= 125;
        humidity -= 6;
        // prevent values less than zero
        if (humidity > 0.0f)
        {
            out = humidity;
        }
    }
    
    return out;
}

/***************************************************************************/
/**
 *	@brief I2C write routine for SHT2x
 *
 *	@Return : Status of the I2C write
 *	@param dev_addr : The device address of the sensor
 *	@param reg_addr : Address of the first register, will data is going to be written
 *	@param reg_data : It is a value hold in the array,
 *		will be used for write the value into the register
 *	@param cnt : The no of byte of data to be write
 *
 ***************************************************************************/
static HumidityErrorCode humidity_write(uint8_t dev_addr, uint8_t reg_addr,
                                        uint8_t *reg_data, uint8_t cnt)
{
    HumidityErrorCode error = HUMIDITY_OK;
	uint8_t tx_buffer[4] = {0};
	uint8_t pos = 0;
	tx_buffer[0] = reg_addr;
	ssp_err_t err;
	UINT status;
    
    (void) ((dev_addr));
    if (reg_data != NULL)
    {
        for (pos = 0; pos < cnt; pos++)
        {
            tx_buffer[pos + 1] = *(reg_data + pos);
        }
    }
	
    // set the slave address
    g_i2c_sensor.p_ctrl->info.slave = SHT2x_ADDRESS;

	// send data via IIC
    // consider extra byte for register address
    err = g_i2c_sensor.p_api->write(g_i2c_sensor.p_ctrl, tx_buffer, (uint32_t)(cnt+1), false);
    // wait for TX to complete
    status = tx_semaphore_get(&g_sensor_i2c_semaphore, HUMIDITY_I2C_TIMEOUT);

    // check for error
    if ((err != SSP_SUCCESS) || (status != TX_SUCCESS))
    {
        error = HUMIDITY_WRITE_ERROR;
    }

    return error;
}

/***************************************************************************/
/**
 *	@brief I2C read routine for SHT2x
 *
 *	@Return : Status of the I2C read
 *	@param dev_addr : The device address of the sensor
 *	@param reg_data : This data read from the sensor, which is hold in an array
 *	@param cnt : The no of data to be read
 *
 ***************************************************************************/
static HumidityErrorCode humidity_read(uint8_t dev_addr, uint8_t *reg_data, uint8_t cnt)
{
    HumidityErrorCode error = HUMIDITY_OK;
	uint8_t rx_buffer[4] = {0};
	uint8_t pos = 0;
	ssp_err_t err;
	UINT status;
	
    (void) ((dev_addr));
    // read I2C data
    if (cnt > sizeof(rx_buffer))
    {
        // buffer not big enough
        error = HUMIDITY_READ_ERROR;
    }
    else
    {
        // set the slave address
        g_i2c_sensor.p_ctrl->info.slave = SHT2x_ADDRESS;
        // read I2C data
        err = g_i2c_sensor.p_api->read(g_i2c_sensor.p_ctrl, rx_buffer, cnt, false);
        // wait for TX to complete
        status = tx_semaphore_get(&g_sensor_i2c_semaphore, HUMIDITY_I2C_TIMEOUT);
        // check for error
        if ((err != SSP_SUCCESS) || (status != TX_SUCCESS))
        {
            error = HUMIDITY_READ_ERROR;
        }
		else
		{
            // data is valid
			for (pos = 0; pos < cnt; pos++)
            {
                *(reg_data + cnt - 1 - pos) = rx_buffer[pos];
            }
		}
    }
    
	return error;
}

/***************************************************************************/
/**
 *	@brief Delay routine
 *
 ***************************************************************************/
static void humidity_delay_msek(uint16_t msek)
{
    tx_thread_sleep(msek/10);
}
