/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#include <stdbool.h>
#include "light.h"
#include "sensor_thread.h"

// Light sensor timeout
#define LIGHT_I2C_TIMEOUT    500

#define LIGHT_MAX_REG    8
#define LIGHT_SLAVE_ADDR (0x44)     // Id of the sensor low bit will
                                    // be used as the Data direction
#define ADC_RESUTION_MAX (65535)

static uint8_t tx_buff[2];
static uint8_t rx_buff[1];

static LightValue_t LightValue = 0;
static uint8_t LuxRange = RANGE_8000;

// forward function declarations
static float GetLightLux(LightValue_t raw);

/***************************************************************************/
/*			light_init - light.h for description			                     */
/***************************************************************************/
LightErrorCode light_init(void)
{
    LightErrorCode error = LIGHT_OK;

    // TODO: validate device by reading known register values
    error = light_power_down ();

    return error;
}

/***************************************************************************/
/*			light_start - light.h for description			                     */
/***************************************************************************/
LightErrorCode light_start(void)
{
    LightErrorCode error = LIGHT_OK;
    // ALS continuous
    error = light_write (REG_COMMANDI, OM_ALS_CONT | INC_16, false);

    if (LIGHT_OK == error)
    {
        error = light_write (REG_COMMANDII, ADC_16 | LuxRange, false);
    }

    return error;
}

/***************************************************************************/
/*			light_power_down - light.h for description			           */
/***************************************************************************/
LightErrorCode light_power_down (void)
{
    LightErrorCode error;

    error = light_write (REG_COMMANDI, OM_POWER_DOWN, false);

    return error;
}

/***************************************************************************/
/*			light_write - light.h for description			                     */
/***************************************************************************/
LightErrorCode light_write(uint8_t reg, uint8_t const value, bool restart)
{
    LightErrorCode error = LIGHT_OK;
    uint16_t count;
    ssp_err_t err;
    UINT status;

    // register must be within range
    if (LIGHT_MAX_REG > reg)
    {
		// register
		tx_buff[0] = reg;
		tx_buff[1] = value;
		if (0xFF == value)
		{
		  // xmit
		  count = 1;
		}
		else
		{
		  count = sizeof(tx_buff);
		}
		// set the slave address
		g_i2c_sensor.p_ctrl->info.slave = LIGHT_SLAVE_ADDR;
		// send data via IIC
	 	err = g_i2c_sensor.p_api->write(g_i2c_sensor.p_ctrl, tx_buff, count, restart);
	 	// wait for TX to complete
	 	status = tx_semaphore_get(&g_sensor_i2c_semaphore, LIGHT_I2C_TIMEOUT);
	 	// check for error
	 	if ((err != SSP_SUCCESS) || (status != TX_SUCCESS))
	 	{
	 	    error = LIGHT_WRITE_ERROR;
	 	}
    }
    else
    {
        error = LIGHT_INT_ERROR;
    }

    return error;
}

LightErrorCode light_read(uint8_t reg, uint8_t *value)
{
	LightErrorCode error = LIGHT_OK;
	ssp_err_t err;
	UINT status;

	// register must be within range
    if (LIGHT_MAX_REG > reg)
    {
		// write out register
		error = light_write(reg, 0xFF, true);
		if (LIGHT_OK == error)
		{
			// read I2C data
		    err = g_i2c_sensor.p_api->read(g_i2c_sensor.p_ctrl, rx_buff, sizeof(rx_buff), false);
		    // wait for TX to complete
		    status = tx_semaphore_get(&g_sensor_i2c_semaphore, LIGHT_I2C_TIMEOUT);
	 		// check for error
	 		if ((err != SSP_SUCCESS) || (status != TX_SUCCESS))
	 		{
				error = LIGHT_READ_ERROR;
				*value = 0xFF;
	 		}
			else
			{
				*value = rx_buff[0];
			}
		}
    }
    else
    {
        error = LIGHT_INT_ERROR;
    }

	return error;
}

/***************************************************************************/
/*			serve_light - light.h for description			               */
/***************************************************************************/
float light_read_lux(void)
{
    float lux = 0;
    LightErrorCode err;

    err = light_read (REG_DATALSB, (uint8_t *)&LightValue);

    if (LIGHT_OK == err)
    {
        err = light_read (REG_DATAMSB, (uint8_t *)&LightValue + 1);
    }
    
    if (LIGHT_OK == err)
    {
        lux = GetLightLux(LightValue);
    }
    
    return lux;
}

/***************************************************************************/
/*			GetLightLux - light.h for description			               */
/***************************************************************************/
static float GetLightLux(LightValue_t raw)
{
    float out;
    uint16_t LuxScale;

    switch (LuxRange)
    {
        case RANGE_125:
            LuxScale = 125;
            break;

        case RANGE_500:
            LuxScale = 500;
            break;

        case RANGE_2000:
            LuxScale = 2000;
            break;

        case RANGE_8000:
            LuxScale = 8000;
            break;

        default:
            break;
    }
    // get a Lux value based on the scale (assumes 16 resolution all the time)
    out = raw;
    out /= ADC_RESUTION_MAX;
    out *= LuxScale;

    return out;
}
