/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#ifndef __LIGHT_H__
#define __LIGHT_H__

#include <stdint.h>

/**
 * @brief Light sensor return codes.
 */
typedef enum
{
	LIGHT_OK = 0,				/**< no error; success */
	LIGHT_WRITE_ERROR,		    /**< error writing data to light sensor */
	LIGHT_READ_ERROR,			/**< error reading data from light sensor */
	LIGHT_BUSY_ERROR,			/**< error light sensor already in use */
	LIGHT_ACK_ERROR,			/**< error acknowledge failure */
	LIGHT_TIMEOUT_ERROR,		/**< error timeout */
	LIGHT_OPEN_ERROR,            /**< error opening light sensor */
	LIGHT_INT_ERROR 			/**< internal light sensor error */

}
LightErrorCode;

// #define LIGHT_SAMPLE_PERIOD   (BSP_TICKS_PER_SEC / 10)

typedef uint16_t LightValue_t;

#define REG_COMMANDI    0   // OP2 OP1 OP0 0 0 FLAG PRST1 PRST0 00h
#define REG_COMMANDII   1   // 0 0 0 0 RES1 RES0 RANGE1 RANGE0 00h
#define REG_DATALSB     2   // D7 D6 D5 D4 D3 D2 D1 D0 00h
#define REG_DATAMSB     3   // D15 D14 D13 D12 D11 D10 D9 D8 00h
#define REG_INT_LT_LSB  4   // TL7 TL6 TL5 TL4 TL3 TL2 TL1 TL0 00h
#define REG_INT_LT_MSB  5   // TL15 TL14 TL13 TL12 TL11 TL10 TL9 TL8 00h
#define REG_INT_HT_LSB  6   // TH7 TH6 TH5 TH4 TH3 TH2 TH1 TH0 FFh
#define REG_INT_HT_MSB  7   // TH15 TH14 TH13 TH12 TH11 TH10 TH9 TH8 FFh

// COMMAND I REG
// Operation Mode
#define OM_POWER_DOWN   (0)
#define OM_ALS_CONT     (5 << 5)
#define OM_IR_CONT      (6 << 5)
// Integration Cycles
#define INC_1           0
#define INC_4           1
#define INC_8           2
#define INC_16          3

// COMMAND II REG
// ADC RESOLUTION DATA WIDTH
#define ADC_16          (0 << 2)
#define ADC_12          (1 << 2)
#define ADC_8           (2 << 2)
#define ADC_4           (3 << 2)
// RANGE/FSR LUX
#define RANGE_125       0
#define RANGE_500       1
#define RANGE_2000      2
#define RANGE_8000      3

/***************************************************************************/
/**
 *	@brief Initialize light sensor.
 *
 ***************************************************************************/
LightErrorCode light_init(void);

/***************************************************************************/
/**
 *	@brief Starts light measurements.
 *
 ***************************************************************************/
LightErrorCode light_start(void);

/***************************************************************************/
/**
 *	@brief Power down the device (for power saving modes)
 *
 ***************************************************************************/
LightErrorCode light_power_down (void);

/***************************************************************************/
/**
 *	@brief Write to the light sensor.
 *
 *	@param reg - sensor register
 *	@param value - value to write
 *
 ***************************************************************************/
LightErrorCode light_write(uint8_t reg, uint8_t const value, bool const restart);

/***************************************************************************/
/**
 *	@brief Read from the light sensor.
 *
 *	@param reg - sensor register
 *	@param value - read value
 *
 ***************************************************************************/
LightErrorCode light_read(uint8_t reg, uint8_t *value);


/***************************************************************************/
/**
 *	@brief Service the update of the light source data
 *
 *	@param None
 *
 ***************************************************************************/
float light_read_lux(void);


#endif /* __LIGHT_H__ */
