/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#include <stdbool.h>
#include "pressure.h"
#include "bmp280.h"
#include "sensor_thread.h"

// Pressure sensor timeout
#define PRESSURE_I2C_TIMEOUT    100

#define	I2C_BUFFER_LEN  24
#define BMP280_ONE_U8X  1

/*----------------------------------------------------------------------------*
 *  struct bmp280_t parameters can be accessed by using bmp280
 *	bmp280_t having the following parameters
 *	Bus write function pointer: BMP280_WR_FUNC_PTR
 *	Bus read function pointer: BMP280_RD_FUNC_PTR
 *	Delay function pointer: delay_msec
 *	I2C address: dev_addr
 *	Chip id of the sensor: chip_id
 *---------------------------------------------------------------------------*/
struct bmp280_t bmp280;

// forward function declarations
static s8 pressure_i2c_setup();
static void BMP280_delay_msek(u16 msek);
static s8  BMP280_I2C_bus_write(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt);
static s8  BMP280_I2C_bus_read(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt);

/***************************************************************************/
/*			pressure_init - pressure.h for description			           */
/***************************************************************************/
PressureErrorCode pressure_init(void)
{
    /* result of communication results*/
	s32 com_rslt = E_BMP280_ERROR;
    /* The variable used to assign the standby time*/
	u8 v_standby_time_u8 = BMP280_INIT_VALUE;
    
    // setup I2C bus according to Bosch BMP280 interface
    pressure_i2c_setup();
    
    // small delay before working on a bus
    BMP280_delay_msek(100);
    
    com_rslt = bmp280_init(&bmp280);
    
    /*	For initialization it is required to set the mode of
	 *	the sensor as "NORMAL"
	 *	data acquisition/read/write is possible in this mode
	 *	by using the below API able to set the power mode as NORMAL*/
	/* Set the power mode as NORMAL*/
	com_rslt += bmp280_set_power_mode(BMP280_NORMAL_MODE);

	/*	For reading the pressure and temperature data it is required to
	 *	set the work mode
	 *	The measurement period in the Normal mode is depends on the setting of
	 *	over sampling setting of pressure, temperature and standby time
	 *
	 *	OSS				pressure OSS	temperature OSS
	 *	ultra low power			x1			x1
	 *	low power				x2			x1
	 *	standard resolution		x4			x1
	 *	high resolution			x8			x2
	 *	ultra high resolution	x16			x2
	 */
	/* The oversampling settings are set by using the following API*/
	com_rslt += bmp280_set_work_mode(BMP280_ULTRA_LOW_POWER_MODE);

    /* This API used to Write the standby time of the sensor input
	 *	value have to be given*/
	 /*	Normal mode comprises an automated perpetual cycling between an (active)
	 *	Measurement period and an (inactive) standby period.
	 *	The standby time is determined by the contents of the register t_sb.
	 *	Standby time can be set using BMP280_STANDBYTIME_125_MS.
	 *	Usage Hint : BMP280_set_standbydur(BMP280_STANDBYTIME_125_MS)*/

	com_rslt += bmp280_set_standby_durn(BMP280_STANDBY_TIME_1_MS);

	/* This API used to read back the written value of standby time*/
	com_rslt += bmp280_get_standby_durn(&v_standby_time_u8);
    
    com_rslt += bmp280_set_oversamp_pressure(BMP280_OVERSAMP_1X);
    com_rslt += bmp280_set_oversamp_temperature(BMP280_OVERSAMP_1X);
    
    return (com_rslt == 0) ? PRESSURE_OK : PRESSURE_INT_ERROR;
}

/***************************************************************************/
/*			pressure_read_pressusre - pressure.h for description		   */
/***************************************************************************/
PressureErrorCode pressure_read_pressusre(float *pressure)
{
    /* result of communication results*/
	s32 com_rslt = E_BMP280_ERROR;
    /* The variable used to read uncompensated pressure*/
	s32 v_data_uncomp_pres_s32 = BMP280_INIT_VALUE;
    *pressure = 0;
    
    /* API is used to read the uncompensated pressure*/
	com_rslt = bmp280_read_uncomp_pressure(&v_data_uncomp_pres_s32);
    if (0 == com_rslt)
    {
        *pressure = (float)bmp280_compensate_pressure_double(v_data_uncomp_pres_s32);
    }
    
    return (com_rslt == 0) ? PRESSURE_OK : PRESSURE_INT_ERROR;
}

/***************************************************************************/
/*			pressure_read_temp - pressure.h for description			       */
/***************************************************************************/
PressureErrorCode pressure_read_temp(float *temp)
{
    /* result of communication results*/
	s32 com_rslt = E_BMP280_ERROR;
    /* The variable used to read uncompensated temperature*/
	s32 v_data_uncomp_tem_s32 = BMP280_INIT_VALUE;
    *temp = 0;
    
    /* API is used to read the uncompensated temperature*/
	com_rslt = bmp280_read_uncomp_temperature(&v_data_uncomp_tem_s32);
    if (0 == com_rslt)
    {
        *temp = (float)bmp280_compensate_temperature_double(v_data_uncomp_tem_s32);
    }
    
    return (com_rslt == 0) ? PRESSURE_OK : PRESSURE_INT_ERROR;
}

/***************************************************************************/
/*			pressure_read_pressure_and_temp - pressure.h for description   */
/***************************************************************************/
PressureErrorCode pressure_read_pressure_and_temp(float *pressure, float *temp)
{
    /* result of communication results*/
	s32 com_rslt = E_BMP280_ERROR;
    s32 pressure_raw = 0;
    s32 temp_raw = 0;
	*temp = BMP280_INIT_VALUE;
	/* The variable used to read real pressure*/
	*pressure = BMP280_INIT_VALUE;
    
    /* API is used to read the true temperature and pressure*/
	/* Input value as uncompensated pressure and temperature*/
	com_rslt = bmp280_read_uncomp_pressure_temperature(&pressure_raw, &temp_raw);
    
    if (0 == com_rslt)
    {
        *pressure = (float)bmp280_compensate_pressure_double(pressure_raw);
        *temp = (float)bmp280_compensate_temperature_double(temp_raw);
    }
    
    return (com_rslt == 0) ? PRESSURE_OK : PRESSURE_INT_ERROR;
}

/***************************************************************************/
/**
 *	@brief Setup I2C bus according to Bosh example template
 *
 ***************************************************************************/
static s8 pressure_i2c_setup()
{
/*--------------------------------------------------------------------------*
 *  By using bmp280 the following structure parameter can be accessed
 *	Bus write function pointer: BMP280_WR_FUNC_PTR
 *	Bus read function pointer: BMP280_RD_FUNC_PTR
 *	Delay function pointer: delay_msec
 *	I2C address: dev_addr
 *--------------------------------------------------------------------------*/
	bmp280.bus_write = BMP280_I2C_bus_write;
	bmp280.bus_read = BMP280_I2C_bus_read;
	bmp280.dev_addr = BMP280_I2C_ADDRESS1;
	bmp280.delay_msec = BMP280_delay_msek;

	return BMP280_INIT_VALUE;
}

/***************************************************************************/
/**
 *	@brief I2C write routine for BMP280
 *
 *	@Return : Status of the I2C write
 *	@param dev_addr : The device address of the sensor
 *	@param reg_addr : Address of the first register, will data is going to be written
 *	@param reg_data : It is a value hold in the array,
 *		will be used for write the value into the register
 *	@param cnt : The no of byte of data to be write
 *
 ***************************************************************************/
static s8  BMP280_I2C_bus_write(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt)
{
    s8 iError = BMP280_INIT_VALUE;
	u8 array[I2C_BUFFER_LEN];
	u8 stringpos = BMP280_INIT_VALUE;
	array[BMP280_INIT_VALUE] = reg_addr;
	ssp_err_t err;
	UINT status;

	for (stringpos = BMP280_INIT_VALUE; stringpos < cnt; stringpos++)
    {
		array[stringpos + BMP280_ONE_U8X] = *(reg_data + stringpos);
	}
    
	// set the slave address
	g_i2c_sensor.p_ctrl->info.slave = dev_addr;
	// send data via IIC
	err = g_i2c_sensor.p_api->write(g_i2c_sensor.p_ctrl, array, (uint32_t)(cnt+1), false);
	// wait for TX to complete
	status = tx_semaphore_get(&g_sensor_i2c_semaphore, PRESSURE_I2C_TIMEOUT);

    // check for error
    if ((err != SSP_SUCCESS) || (status != TX_SUCCESS))
    {
        iError = -1;
	}

    return iError;
}

/***************************************************************************/
/**
 *	@brief I2C read routine for BMP280
 *
 *	@Return : Status of the I2C read
 *	@param dev_addr : The device address of the sensor
 *	@param reg_addr : Address of the first register, will data is going to be read
 *	@param reg_data : This data read from the sensor, which is hold in an array
 *	@param cnt : The no of data to be read
 *
 ***************************************************************************/
static s8  BMP280_I2C_bus_read(u8 dev_addr, u8 reg_addr, u8 *reg_data, u8 cnt)
{
    s8 iError = BMP280_INIT_VALUE;
	u8 array[I2C_BUFFER_LEN] = {BMP280_INIT_VALUE};
	u8 stringpos = BMP280_INIT_VALUE;
	array[BMP280_INIT_VALUE] = reg_addr;
	ssp_err_t err;
	UINT status;
	
	// set the slave address
	g_i2c_sensor.p_ctrl->info.slave = dev_addr;
	// send data via IIC
	err = g_i2c_sensor.p_api->write(g_i2c_sensor.p_ctrl, array, 1, true);
	// wait for TX to complete
	status = tx_semaphore_get(&g_sensor_i2c_semaphore, PRESSURE_I2C_TIMEOUT);
    
    // read I2C data
    if ((cnt > sizeof(array)) || (err != SSP_SUCCESS) || (status != TX_SUCCESS))
    {
        // buffer not big enough
        iError = -1;
    }
    else
    {
        // read I2C data
        err = g_i2c_sensor.p_api->read(g_i2c_sensor.p_ctrl, array, cnt, false);
        // wait for TX to complete
        status = tx_semaphore_get(&g_sensor_i2c_semaphore, PRESSURE_I2C_TIMEOUT);
        // check for error
        if ((err != SSP_SUCCESS) || (status != TX_SUCCESS))
        {
            iError = -1;
        }
		else
		{
            // data is valid
			for (stringpos = BMP280_INIT_VALUE; stringpos < cnt; stringpos++)
            {
                *(reg_data + stringpos) = array[stringpos];
            }
		}
    }
    
	return iError;
}

/***************************************************************************/
/**
 *	@brief Delay routine
 *
 ***************************************************************************/
static void BMP280_delay_msek(u16 msek)
{
    tx_thread_sleep(msek/10);
}
