/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
#include "sensor_thread.h"
#include "ble_host_thread.h"
#include "application_dispatcher_thread.h"
#include "common.h"
#include "light.h"
#include "pressure.h"
#include "humidity.h"
#include "x_print.h"

extern activity_data g_activity_data;

void sensor_thread_entry(void);
void sensor_i2c_callback(i2c_callback_args_t * p_args);

/* Sensor Thread entry function */
void sensor_thread_entry(void)
{
    uint32_t events;
    ssp_err_t err;
    LightErrorCode lightError;
    float light = 0.0f;
    float pressure = 0.0f;
    float temperature = 0.0f;
    float humidity = 0.0f;
    uint16_t light_value = 0;
    uint16_t pressure_value = 0;
    uint16_t temperature_value = 0;
    uint16_t humidity_value = 0;

    // Wait for BLE ready event
    tx_event_flags_get(&g_ble_ready_event, 0x01, TX_OR, &events, TX_WAIT_FOREVER);

    // open up I2C device
    err = g_i2c_sensor.p_api->open(g_i2c_sensor.p_ctrl, g_i2c_sensor.p_cfg);
    if (err != SSP_SUCCESS)
    {
        x_printf("ERROR: %d", err);
    }

    // Initialize pressure sensor
    if (PRESSURE_OK != pressure_init())
    {
        x_printf("ERROR: %s", "pressure init");
    }

    // Initialize light sensor
    lightError = light_init();
    if (LIGHT_OK == lightError)
    {
        lightError = light_start();
    }

    // Initialize humidity sensor
    if (HUMIDITY_OK != humidity_init())
    {
        x_printf("ERROR: %s", "humidity init");
    }

    // let everything settle
    tx_thread_sleep (100);

    while (1)
    {
        // trigger humidity reading
        humidity_trigger();

        // read sensor data every 1 second
        tx_thread_sleep (100);

        // Light
        light = light_read_lux();
        light_value = (uint16_t) (light * 10.0f);

        // Pressure
        if (pressure_read_pressure_and_temp(&pressure, &temperature) == PRESSURE_OK)
        {
            pressure_value = (uint16_t)pressure;
            temperature_value = (uint16_t) (temperature * 10.0f);
        }

        // Humidity
        if (humidity_read_humidity(&humidity) == HUMIDITY_OK)
        {
            humidity_value = (uint16_t) (humidity * 10.0f);
        }

        // update data
        tx_mutex_get(&g_data_mutex, TX_WAIT_FOREVER);
        g_activity_data.humidity = humidity_value;
        g_activity_data.light = light_value;
        g_activity_data.pressure = pressure_value;
        g_activity_data.temperature = temperature_value;
        tx_mutex_put(&g_data_mutex);
    }
}

/*
 * Call back function for sensor based I2C devices
 */
void sensor_i2c_callback(i2c_callback_args_t * p_args)
{
    (void) ((p_args));
    tx_semaphore_ceiling_put(&g_sensor_i2c_semaphore, 1);
}


