package com.xaplos.hcsolution;

import java.lang.ref.WeakReference;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;
import java.util.Timer;
import java.util.TimerTask;

import android.app.Service;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothClass;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothGatt;
import android.bluetooth.BluetoothGattCallback;
import android.bluetooth.BluetoothGattCharacteristic;
import android.bluetooth.BluetoothGattDescriptor;
import android.bluetooth.BluetoothGattService;
import android.bluetooth.BluetoothManager;
import android.bluetooth.BluetoothProfile;
import android.content.Context;
import android.content.Intent;
import android.content.pm.PackageManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.IBinder;
import android.os.Looper;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.Log;
import android.os.Build;

public class BLEService extends Service {
	private final static String TAG = "BLEService";
	private final Messenger mMessenger = new Messenger(new IncomingHandler(this));
    private final Handler mHandler = new Handler();
    private Messenger mClient;
    private final HashMap<String, BluetoothDevice>mDiscoveredDevices = new HashMap<>();

	private final BLEDevice mDevice = new ActivityDemoDevice();
    private BluetoothAdapter mAdapter;
    private final BluetoothAdapter.LeScanCallback mLeScanCallback = new BLEServiceLeScanCallback();
	private final BluetoothGattCallback mGattCallback = new BLEServiceBluetoothGattCallback();

	private final boolean mForceEmulation = false;
	private final Timer mEmulationTimer = new Timer();

	private final Timer mServicesTimer = new Timer();
	private boolean mServicesDiscovered = false;

    // Key string for the bundle use to represent a found device
    public final static String KEY_NAME = "name";
    public final static String KEY_ADDRESS = "address";
    public final static String KEY_RSSI = "rssi";
    public final static String KEY_CLASS = "class";

	/**
	 * Requests coming from the Activity
	 */
	public enum Request {
        START_SCANNING,
		SCAN,
        WRITE_DESCRIPTOR,
        CONNECT,
        DISCONNECT
	}

	/**
	 * Messages to send to the Activity
	 */
	public enum Response {
		NEED_BLUETOOTH,
		NOTIFICATION,
		DIAGNOSTIC,
	}

	/**
	 * Diagnostic messages
	 */
	public enum Diagnostic {
		SCANNING,
		DEVICE_FOUND,
		DEVICE_NOT_FOUND,
		DEVICE_CONNECTED,
		DEVICE_DISCONNECTED,
        DEVICE_NOT_SUPPORTED,
		BLE_NOT_SUPPORTED,
		BLE_NOT_ENABLED
	}
	
	/**
	 * Handle messages from activities
	 */ 
	static class IncomingHandler extends Handler {
		private static final String TAG = "IncomingHandler";
		private final WeakReference<BLEService> mService;
		
		public IncomingHandler(BLEService service) {
			mService = new WeakReference<>(service);
		}
		
		@Override
		public void handleMessage(Message msg) {
			Request action = Request.values()[msg.what];
			Log.d(TAG, "handleMessage " + action.toString());
			BLEService service = mService.get();

			if (service == null) {
				Log.e(TAG, "BLEService is null");
                return;
			}
			
			service.mClient = msg.replyTo;
            Bundle bundle = msg.getData();

			switch(action) {
			//
			// The first message from the activity. Kick everything off.
			//
			case START_SCANNING:
				service.handleScanRequest();
				break;
			
			//
			// The user enabled bluetooth per the activity's request.
			//
			case SCAN:
				service.handleRequestScan();
				break;

            case WRITE_DESCRIPTOR:
                service.handleWriteRequest(bundle);
                break;

            case CONNECT:
                service.handleConnectRequest(bundle);
                break;

            case DISCONNECT:
                service.disconnect();
                break;
			}
		}
	}

    private void disconnect() {
        BluetoothGatt gatt = mDevice.getGatt();

        if (gatt != null) {
            gatt.disconnect();
        }
        else {
            Log.e(TAG, "can't disconnect because gatt is null");
        }
    }

    private void handleConnectRequest(Bundle bundle) {
        Log.d(TAG, "handleConnectRequest " + bundle);
        String address = bundle.getString("address");

        if (mDiscoveredDevices.containsKey(address)) {
            mDevice.resetWatchdog();
            BluetoothDevice device = mDiscoveredDevices.get(address);
            Log.d(TAG, "connecting to " + mDevice.getDeviceName());
            Handler handler = new Handler(Looper.getMainLooper());
            ConnectRunnable runnable = new ConnectRunnable(device);
            handler.post(runnable);
        }
        else {
            Log.e(TAG, "requested a connection to an unknown device");
        }
    }

	private void handleScanRequest() {
        mDiscoveredDevices.clear();

        if (setupEmulation()) {
            Log.i(TAG, "bluetooth emulated");
        }
        else {
            if (bleSetup()) {
                // BLE is fully enabled
                bleStartScan();
            } else {
                Log.e(TAG, "no bluetooth?");
            }
        }
	}
	
	private void handleRequestScan() {
		bleStartScan();
	}

    private void handleWriteRequest(Bundle bundle) {
        String uuidString = bundle.getString("uuid");
        byte[] data = bundle.getByteArray("bytes");
		String hex = "";

        if (data == null) {
            Log.e(TAG, "handleWriteRequest: data is null");
            return;
        }

		for (int i = 0; i < data.length; i++) {
            if (i > 0) {
                hex += ",";
            }

			hex += Integer.toHexString(data[i]);
		}

        Log.d(TAG, "handleWriteRequest(" + uuidString + ", bytes{" + hex + "})");
        UUID uuid = UUID.fromString(uuidString);

        //noinspection ConstantConditions
        if (mDevice != null) {
            mDevice.writeBytesToDescriptor(uuid, data);
        }
        else {
            Log.e(TAG, "null device");
        }
    }

	/**
	 * Exception catching wrapper for Messenger.send()
	 * @param msg
	 */
	private void sendMessage(Message msg) {
		if (mClient == null) {
            /*
             * This can happen when the main activity is closed, leaving the
             * service running with no GUI.
             */
			Log.w(TAG, "no client for message: " + responseDebug(msg));
		}
		else {
			try {
				mClient.send(msg);
			}
			catch (RemoteException e) {
				e.printStackTrace();
				mClient = null;
			}
		}
	}
	
	/**
	 * Send a message to the client Activity. The bundle may contain info (activity rate, etc) from a BLE service.
	 * @param response
	 * @param bundle
	 */
	private void sendResponse(Response response, Bundle bundle) {
		Message msg = Message.obtain(null, response.ordinal());
		msg.setData(bundle);
		sendMessage(msg);
	}
	
	/**
	 * Send a message to the client Activity. The message contains info on connection state and errors. 
	 * @param diagnostic
	 */
	private void sendDiagnostic(Diagnostic diagnostic, Bundle data) {
		Message msg = Message.obtain(null, Response.DIAGNOSTIC.ordinal(), diagnostic.ordinal(), 0);

        if (data != null) {
            msg.setData(data);
        }

		sendMessage(msg);
	}

    private void sendDiagnostic(Diagnostic diagnostic) {
        sendDiagnostic(diagnostic, null);
    }
        @Override
	public int onStartCommand(Intent intent, int flags, int startId) {
		Log.d(TAG, "onStartCommand(" + intent + ", " + flags + ", " + startId);

		if (intent == null) {
			Log.w(TAG, "null intent: restart");
		}
		
		return START_STICKY;
	}

	@Override
	public IBinder onBind (Intent intent) {
		// TODO set mClient to null?
		Log.d(TAG, "onBind");
		return mMessenger.getBinder();
	}
	
	@Override
	public boolean onUnbind(Intent intent) {
		Log.d(TAG, "unUnbind: " + intent.toString());
		return super.onUnbind(intent);
	}

	@Override
	public void onCreate () {
		Log.d(TAG, "onCreate");
	}
	
	@Override
	public void onDestroy () {
		Log.d(TAG, "onDestroy");
		bleShutdown();
	}

    /*
     * Some devices, including Samsung, require connectGatt to be run from the main thread.
     */
    private class ConnectRunnable implements Runnable {
        private final BluetoothDevice mConnectDevice;

        ConnectRunnable(BluetoothDevice device) {
            mConnectDevice = device;
        }

        @Override
        public void run() {
            Log.d(TAG, "connectRunnable calling connectGatt");
            mServicesDiscovered = false;
            mServicesTimer.schedule(new ServicesTimerTask(), 5000);
            BluetoothGatt gatt = mConnectDevice.connectGatt(BLEService.this, false, mGattCallback);
            mDevice.setGatt(gatt);
        }
    }

    private class ServicesTimerTask extends TimerTask {
        private final String TAG = "ServicesTimerTask";

        @Override
        public void run() {
            if (mServicesDiscovered) {
                Log.d(TAG, "services already discovered");
            }
            else {
                Log.d(TAG, "onServicesDiscovered was not called");
                BluetoothGatt gatt = mDevice.getGatt();

                if (gatt != null) {
                    startNotifications(gatt);
                }
                else {
                    Log.e(TAG, "gatt is null, cannot start notifications");
                }
            }
        }
    }

    private class BLEServiceLeScanCallback implements BluetoothAdapter.LeScanCallback {
    	/**
    	 * If the device is something we want to talk to, connect to it. 
    	 */
        @Override
        public void onLeScan(final BluetoothDevice device, int RSSI, byte[] scanRecord) {
       	    Log.d(TAG, "onLeScan: " + device.toString() + " (" + device.getName() + ")");
            String address = device.getAddress();

            // Cache the device
            if (!mDiscoveredDevices.containsKey(address)) {
                mDiscoveredDevices.put(address, device);
            }

            // Tell the UI what we found
            Bundle bundle = new Bundle();
            bundle.putString(KEY_NAME, device.getName());
            bundle.putString(KEY_ADDRESS, device.getAddress());
            bundle.putInt(KEY_RSSI, RSSI);
            BluetoothClass deviceClass = device.getBluetoothClass();

            if (deviceClass != null) {
                bundle.putString(KEY_CLASS, deviceClass.toString());
            }
            else {
                bundle.putString(KEY_CLASS, "");
            }

			sendDiagnostic(Diagnostic.DEVICE_FOUND, bundle);
        }
	}
	
	/**
	 * Set up the BluetoothAdapter
	 */
    private boolean bleSetup() {
        //noinspection ConstantConditions
        if (mForceEmulation) {
    		Log.w(TAG, "emulation is on: not using real bluetooth");
    		return false;
    	}
    	
		if (!getPackageManager().hasSystemFeature(PackageManager.FEATURE_BLUETOOTH_LE)) {
			sendDiagnostic(Diagnostic.BLE_NOT_SUPPORTED);
			return false;
		}
		
		final BluetoothManager bluetoothManager = (BluetoothManager) getSystemService(Context.BLUETOOTH_SERVICE);
		mAdapter = bluetoothManager.getAdapter();
		
		if (null == mAdapter) {
			sendDiagnostic(Diagnostic.BLE_NOT_ENABLED);
			return false;
		}
		
		if (!mAdapter.isEnabled()) {
			sendResponse(Response.NEED_BLUETOOTH, null);
			return false;
		}
		
		return true;
	}
	
    /**
     * Close all GATT connections, notifications and scans
     */
    private void bleShutdown() {
    	if (mAdapter != null) {
    		mAdapter.stopLeScan(mLeScanCallback);
    	}

        //noinspection ConstantConditions
        if (mDevice != null) {
    		BluetoothGatt gatt = mDevice.getGatt();
    		
	    	if (gatt != null) {
				gatt.close();
				mDevice.setGatt(null);
	    	}
    	}
    }
    
    /**
     * Start scanning for BLE devices
     */
	private void bleStartScan() {
		Log.d(TAG, "bleStartScan");
		sendDiagnostic(Diagnostic.SCANNING);

        mHandler.postDelayed(new BleScanTimeoutHandler(), 10000);
        mAdapter.stopLeScan(mLeScanCallback);
        
        if (mAdapter.isDiscovering()) {
        	Log.w(TAG, "cancelling discovery");
        	mAdapter.cancelDiscovery();
        }
        
        mAdapter.startLeScan(mLeScanCallback);		
	}

	/**
	 * This is run in the BLE thread after a set period of time. 
	 */
	private class BleScanTimeoutHandler implements Runnable {
        @Override
		public void run() {
        	Log.d(TAG, "stopping scan after delay");
            mAdapter.stopLeScan(mLeScanCallback);

            if (mDiscoveredDevices.size() == 0) {
            	sendDiagnostic(Diagnostic.DEVICE_NOT_FOUND);
            }
		}
	}

    private final ArrayList<BLECharacteristic>mNotificationStartList = new ArrayList<>();

    private void startNotifications(BluetoothGatt gatt) {
        //
        // Create a queue of characteristics for notification
        //
        mNotificationStartList.clear();

        for (BLECharacteristic characteristic : mDevice.getCharacteristics()) {
            if (characteristic.hasNotifications()) {
                mNotificationStartList.add(characteristic);
            }
        }

        //
        // Start notifications for the first characteristic in the queue
        //
        startNextNotification(gatt);
    }

    private void startNextNotification(BluetoothGatt gatt) {
        if (mNotificationStartList.size() == 0) {
            Log.d(TAG, "startNextNotification: queue emptied");
            return;
        }

        UUID demoService = mDevice.getServiceUuid();
        BluetoothGattService service = gatt.getService(demoService);

        if (service != null) {
            BLECharacteristic deviceCharacteristic = mNotificationStartList.remove(0);
            BluetoothGattCharacteristic characteristic = service.getCharacteristic(deviceCharacteristic.getCharacteristicUuid());

            if (characteristic != null) {
                Log.d(TAG, "startNextNotification: for " + deviceCharacteristic.getCharacteristicName() + ": " + characteristic.getUuid().toString());
                mDevice.setCharacteristicNotification(characteristic, true);
            }
        }
        else {
            Log.e(TAG, "the service does not exist on the device: " + demoService.toString());
            sendDiagnostic(Diagnostic.DEVICE_NOT_SUPPORTED);
        }

        //
        // If that works, onDescriptorWrite will be called eventually
        //
    }

    private class BLEServiceBluetoothGattCallback extends BluetoothGattCallback {
		private final static String TAG = "GattCallback";

        /**
		 * Observe GATT connection changes
		 */
		@Override
		public void onConnectionStateChange(BluetoothGatt gatt, int status, int newState) {
			switch(newState) {
			case BluetoothProfile.STATE_CONNECTED:
				Log.d(TAG, "connected");
                Bundle bundle = new Bundle();
				sendDiagnostic(Diagnostic.DEVICE_CONNECTED, bundle);

				Log.d(TAG, "calling discoverServices");

				if (!gatt.discoverServices()) {
					Log.e(TAG, "discoverServices failed");
				}
				
				break;
				
			case BluetoothProfile.STATE_CONNECTING:
				Log.d(TAG, "connecting");
				break;
				
			case BluetoothProfile.STATE_DISCONNECTED:
				sendDiagnostic(Diagnostic.DEVICE_DISCONNECTED);
				Log.d(TAG, "disconnected");
				break;
				
			case BluetoothProfile.STATE_DISCONNECTING:
				Log.d(TAG, "disconnecting");
				break;
			}
		}

		/**
		 * If we are interested in the newly found service, enable notifications for it
		 */
        @Override
        public void onServicesDiscovered(BluetoothGatt gatt, int status) {
            Log.d(TAG, "onServicesDiscovered status: " + status);
            mServicesDiscovered = true;

			if (status == BluetoothGatt.GATT_SUCCESS) {
                startNotifications(gatt);
            }
            else {
                Log.e(TAG, "service discovery failed");
            }
        }

        @Override
        public void onDescriptorWrite(BluetoothGatt gatt, BluetoothGattDescriptor descriptor, int status) {
            Log.d(TAG, "onDescriptorWrite " + descriptor.getUuid().toString() + " " + status);
            startNextNotification(gatt);
            super.onDescriptorWrite(gatt, descriptor, status);
        }

        @Override
        public void onCharacteristicWrite(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
            Log.d(TAG, "onCharacteristicWrite(" + characteristic.getUuid().toString() + ") status " + status);
            super.onCharacteristicWrite(gatt, characteristic, status);
        }

        /**
         * Monitor the results of characteristic reads
         */
        @Override
        public void onCharacteristicRead(BluetoothGatt gatt, BluetoothGattCharacteristic characteristic, int status) {
        	Log.d(TAG, "onCharacteristicRead: " + characteristic.toString() + " status: " + status);
        	
            if (status != BluetoothGatt.GATT_SUCCESS) {
            	Log.e(TAG, "error reading");
            }
        }

        /**
         * Send the parsed results of characteristic notification values to the client Activity.
		 * This is where the bluetooth notification data enters our service.
         */
        @Override
        public void onCharacteristicChanged (BluetoothGatt gatt, BluetoothGattCharacteristic characteristic) {
            UUID uuid = characteristic.getUuid();
            BLECharacteristic deviceCharacteristic = mDevice.getCharacteristic(uuid);
            byte[] value = characteristic.getValue();
            String valueString = "";
            mDevice.petWatchdog();

            for (int i = 0; i < value.length; i++) {
                if (i > 0) {
                    valueString += ", ";
                }

                valueString += Integer.toHexString(value[i] & 0xff);
            }

            Log.d(TAG, uuid.toString() + " " + deviceCharacteristic.getCharacteristicName() + " = {" + valueString + "}");
        	Bundle bundle = deviceCharacteristic.parseCharacteristicValue(characteristic);
        	sendCharacteristicToApp(bundle);
        }
	}

	private void sendCharacteristicToApp(Bundle bundle) {
    	sendResponse(Response.NOTIFICATION, bundle);
	}

	private boolean setupEmulation() {
        // This is a kludge to test whether we are in the simulation environment.
        boolean isEmulator = Build.HARDWARE.contains("goldfish");
        boolean isEmulationOn = false;

        if (isEmulator) {
            Log.w(TAG, "virtual device detected: " + Build.HARDWARE);
        }

        //noinspection PointlessBooleanExpression,ConstantConditions
        if (mForceEmulation || isEmulator) {
            startEmulation();
            isEmulationOn = true;
		}

        return isEmulationOn;
	}

    private void startEmulation() {
        long delay = 1000;
        mEmulationTimer.scheduleAtFixedRate(mEmulationTask, delay, delay);
    }

    @SuppressWarnings("unused")
    private void stopEmulation() {
        mEmulationTimer.cancel();
    }

    /*
     * Emulate notifications to test the GUI under the Android simulator
     */
    private final TimerTask mEmulationTask = new TimerTask() {
        @Override
        public void run() {
            for (BLECharacteristic characteristic : mDevice.getCharacteristics()) {
                if (characteristic.hasNotifications()) {
                    Bundle bundle = characteristic.emulateCharacteristic();
                    sendCharacteristicToApp(bundle);
                }
            }
        }
    };

    /**
	 * Explain the response message as a string 
	 */
	private String responseDebug(Message message) {
		StringBuilder builder = new StringBuilder();
		Response response = Response.values()[message.what];
		builder.append(response.toString());
		
		if (Response.DIAGNOSTIC == response) {
			Diagnostic diagnostic = Diagnostic.values()[message.arg1];
			builder.append(": ").append(diagnostic.toString());
		}
		
		return builder.toString();
	}
	
	/**
	 * Log.d the characteristic
	 * @param characteristics
	 */
	private void dumpCharacteristics(List<BluetoothGattCharacteristic> characteristics) {
		for (BluetoothGattCharacteristic characteristic : characteristics) {
			UUID charId = characteristic.getUuid();
			Log.d(TAG, "characteristic: " + charId.toString());
		}
	}

	/**
	 * Log.d the gatt service and its characteristics
	 * @param services
	 */
	@SuppressWarnings("unused")
	private void dumpServices(List<BluetoothGattService> services) {
    	for (BluetoothGattService service : services) {
    		Log.d(TAG, "service: " + service.getUuid());
    		List<BluetoothGattCharacteristic> characteristics = service.getCharacteristics();
    		dumpCharacteristics(characteristics);
    	}
	}
}
