/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
 * x_print.c
 *
 *  Created on: Mar 23, 2016
 *      Author: alejandro
 ****************************************************************************************
 */
#include "x_print.h"
#include "bsp_api.h"
#include "tx_api.h"
#include "util.h"

/* This helps to debug using the Renesas Virtual Console and other USBX (ex CDC over USB)
 * at the moment Virtual Console seem quite slow ... you can avoid directing
 * output to it by calling x_print_njtag, it will disable redirection through JTAG
 *
 * Expecting the use of Communication Framework to be added under components
 * (g_sf_comms is the default name)
 * please refer to the app note below for more details on using CDC over USB
 * http://knowledgebase.renesas.com/Renesas_Synergy_Platform/Renesas_Synergy_Knowledge_Base/Serial_communication_using_USBX
 */
extern const sf_comms_instance_t g_sf_comms;

// lines up to 128 bytes in length ... seem like enough
#define LOCAL_BUFF_SIZE 256
#define PRINTF_QUEUE_SIZE 128

typedef struct
{
    char buff[LOCAL_BUFF_SIZE];
    int count;
}x_printf_buff_t;;

typedef struct
{
    TX_QUEUE ctrl;
    uint32_t storage[PRINTF_QUEUE_SIZE];
    x_printf_buff_t scratchpad;
    uint32_t qdata;
    x_printf_buff_t service;   // scratch pad for service call
}x_printf_isr_context_t;

typedef struct
{
    TX_THREAD watchdog;
    uint8_t stack[256];
    TX_THREAD *caller;
    TX_SEMAPHORE start;
    int count;      // track how many writes have taken place
    int timeout;    // tick timeout
    bool disable;   // disable any writes
}x_printf_write_info_t;

typedef struct
{
    bool init;
    bool jtag;
    x_printf_buff_t pad_main;   // main scratch pad
    x_printf_buff_t pad_sec;    // secondary scrathpad
    x_printf_isr_context_t isr;
    const char *msg_trunc;
    const int msg_trunc_len;
    sf_comms_ctrl_t *ctrl;
    sf_comms_api_t const *api;
    TX_MUTEX lock;
    TX_THREAD thread;
    uint8_t stack[1024];
    x_printf_write_info_t write;
}x_printf_context_t;

static const char msg_truncated[] = {" ...>\n\r"};

static x_printf_context_t x_printf_context =
{
    .msg_trunc = msg_truncated,
    .msg_trunc_len = sizeof (msg_truncated),
	.write.disable = false,
	.write.timeout = (TX_TIMER_TICKS_PER_SECOND * CDC_WRITE_TIMEOUT_SEC),

};

// JTAG redirection
static void (*jtag_printf) (const char *format, ...) = NULL;

/*
 * forward declarations for helpers
 */
static int x_printf_isr_service(void);
static void x_printf_isr_init (x_printf_context_t *c);
static void x_printf_isr_heading (x_printf_context_t *c);
static void x_printf_queue_str (x_printf_context_t *c, const char *str, int len);
static void x_printf_thread_create (x_printf_context_t *c);
static void x_printf_thread (ULONG thread_input);
static void x_printf_thread_watchdog (ULONG thread_input);
static ssp_err_t x_printf_flush_x(x_printf_context_t *c, const char *buff, int len);

/***
 * @brief refer to prototype for details
 */
void x_print_njtag (void)
{
    x_print_init (false);
}

/***
 * @brief refer to prototype for details
 */
void x_print_init (bool jtag_use)
{
    x_printf_context_t *c = &x_printf_context;
    //
    x_printf_user_init ();
    // create the mutex
    tx_mutex_create (&c->lock, "app level logger mutex", TX_NO_INHERIT);
    // is jtag tool connected?
    bool jtag_connected = CoreDebug->DHCSR & CoreDebug_DHCSR_C_DEBUGEN_Msk;
    //
    if (jtag_use && jtag_connected)
    {
        // connected to the JTAG
        initialise_monitor_handles ();
        // we have debugger connected ... redirect this to the Renesas virtual console
        jtag_printf = (void*)printf;
        //
        c->jtag = true;
    }
    //
    c->ctrl = g_sf_comms.p_ctrl;
    c->api = g_sf_comms.p_api;
    //
    x_printf_isr_init (c);

    x_printf_thread_create (c);
    //
    c->init = true;
}

/***
 * @brief refer to prototype for details
 */
void x_printf_isr(const char *format, ...)
{
    x_printf_context_t *c = &x_printf_context;
    va_list v;
    int length;
    char *buff = c->isr.scratchpad.buff;

    // show that this log is coming from an ISR
    x_printf_isr_heading (c);
    // not much we can do in this context ...
    va_start(v, format);
    length = vsnprintf(buff, LOCAL_BUFF_SIZE, format, v);
    va_end(v);
    // queue the text
    x_printf_queue_str (c, buff, MIN (length, LOCAL_BUFF_SIZE));
    // truncated?
    if (length >= LOCAL_BUFF_SIZE)
    {
        // hint of the truncation
        x_printf_queue_str (c, c->msg_trunc, sizeof (c->msg_trunc));
    }
}

/***
 * @brief refer to prototype for details
 */
ssp_err_t x_printf_flush(void)
{
    x_printf_context_t *c = &x_printf_context;
    ssp_err_t err;

    if (c->write.disable)
    {
        // reset
        c->pad_main.count = 0;
        // just for clarity
        c->pad_main.buff[0] = '\0';
        // no need to do anthing else
        return SSP_SUCCESS;
    }

    // get the lock
    tx_mutex_get (&c->lock, TX_WAIT_FOREVER);
    //
    err = x_printf_flush_x (c, c->pad_main.buff, c->pad_main.count);
    //
    if (SSP_SUCCESS == err)
    {
        // reset
        c->pad_main.count = 0;
        // just for clarity
        c->pad_main.buff[0] = '\0';
    }
    else
    {
        // NOTE:
        // handle error case here
    }

    // done
    tx_mutex_put (&c->lock);

    return err;
}

void x_print_raw_flush(void)
{
    x_printf_flush ();
}

void x_print_raw(uint8_t *data, int count)
{
    x_printf_context_t *c = &x_printf_context;
    x_printf_buff_t *pad = &c->pad_main;
    // get the lock
    tx_mutex_get (&c->lock, TX_WAIT_FOREVER);
    // how much can we fit in this buffer
    int chunk = MIN (count, sizeof (pad->buff) - pad->count);
    int left = count - chunk;
    // copy it over
    memcpy (&c->pad_main.buff[c->pad_main.count], data, (size_t)chunk);
    //
    c->pad_main.count += chunk;
    //
    if (c->pad_main.count == sizeof (pad->buff))
    {
        // flush now
        x_printf_flush ();
    }
    //
    tx_mutex_put (&c->lock);
    // anything left?
    if (left > 0)
    {
        // put some more
        x_print_raw (data + chunk, count - chunk);
    }
}

/***
 * @brief refer to prototype for details
 */
void x_printf(const char *format, ...)
{
    va_list v;
    int length;
    bool is_truncated;
    x_printf_context_t *c = &x_printf_context;
    x_printf_buff_t *pad = &c->pad_sec;
    char *buff = pad->buff;
    ssp_err_t err = SSP_SUCCESS;

    if (SCB->ICSR & SCB_ICSR_VECTACTIVE_Msk)
    {
        // this routine should not be called in Interrupt context
        // queue a warning and return
        x_printf_isr ("DO NOT CALL \"%s\" from INTERRUPT context! USE \"x_printf_isr\" instead.\n\r", __FUNCTION__);
        return;
    }
    // get the lock
    tx_mutex_get (&c->lock, TX_WAIT_FOREVER);
    //
    va_start(v, format);
    length = vsnprintf(buff, LOCAL_BUFF_SIZE, format, v);
    va_end(v);
    //
    is_truncated = length >= LOCAL_BUFF_SIZE;
    // sending through JTAG?
    if (jtag_printf)
    {
        // yes
        jtag_printf ("%s", buff);
        // was the line truncated?
        if (is_truncated)
        {
            // somehow show the truncation
            jtag_printf ("%s", " ...>\n");
        }
    }

    if (c->write.disable)
    {
        // NOTE:
        // no need to do anything else ... no way to send this through USB
    }
    else
    {
        // did not fit?
        if (is_truncated)
        {
            // adjust the length to the string length
            length = LOCAL_BUFF_SIZE - 1;
        }
        //
        pad->count = length;
        // does main has room for the secondary scratchpad?
        if (c->pad_main.count + pad->count < (int)sizeof (c->pad_main.buff))
        {
            // copy over
            memcpy (&c->pad_main.buff[c->pad_main.count], pad->buff, (size_t)pad->count);
            // keep track
            c->pad_main.count += pad->count;
        }
        else
        {
            // main needs to be flushed
            // send it through the terminal also
            err = x_printf_flush ();

            if (SSP_SUCCESS == err)
            {
                // copy over pad to main
                memcpy (&c->pad_main.buff, pad->buff, (size_t)pad->count);
                // size
                c->pad_main.count = pad->count;
            }
            else
            {
                // NOTE:
                // handle error condition here
            }

        }
        // reset this
        pad->count = 0;
        // just for clarity
        pad->buff[0] = 0;
    }
    // done
    tx_mutex_put (&c->lock);
}

/*
 * local helpers below this line
 */
static int x_printf_isr_service(void)
{
    x_printf_context_t *c = &x_printf_context;
    UINT status;
    uint32_t data;
    ssp_err_t err = SSP_SUCCESS;
    int count = 0;
    // has the logger context been initialized?
    if (c->init)
    {
        // yes, get the scratchpad for servicing ISR logging requests
        x_printf_buff_t *pad = &c->isr.service;
        char *p = pad->buff;
        // reset
        pad->count = 0;
        bool flush = false;

        do
        {
            status = tx_queue_receive(&c->isr.ctrl, &data, TX_NO_WAIT);

            switch (status)
            {
                case TX_QUEUE_EMPTY:
                    flush = pad->count > 0;
                    break;

                case TX_SUCCESS:
                    memcpy (p, &data, sizeof (data));
                    p += sizeof (data);
                    pad->count += (int)sizeof (data);
                    // we filled local
                    if (pad->count >= (int)sizeof (c->isr.service.buff))
                    {
                        // yes
                        flush = true;
                    }

                    break;

                default:
                    // NOTE:
                    // handle other errors here
                    break;
            }
            // do we need to flush?
            if (flush && pad->count)
            {
                // track
                count += pad->count;
                // get the lock
                tx_mutex_get (&c->lock, TX_WAIT_FOREVER);
                // send out
                err = x_printf_flush_x (c, c->isr.service.buff, pad->count);
                // release
                tx_mutex_put (&c->lock);
                // reset
                pad->count = 0;
            }
        }
        while (TX_SUCCESS == status && SSP_SUCCESS == err);
    }

    return count;
}

static void x_printf_isr_init (x_printf_context_t *c)
{
    UINT status;
    // create the queue
    status = tx_queue_create(&c->isr.ctrl, "x_printf isr queue", TX_1_ULONG,
                             (VOID *) c->isr.storage, sizeof (c->isr.storage));

    if (TX_SUCCESS == status)
    {
        //
    }
    else
    {
        // NOTE:
        // handle error condition
    }
}

static void x_printf_queue_str (x_printf_context_t *c, const char *str, int len)
{
    UINT status = TX_SUCCESS;
    int chunk = MIN (len, LOCAL_BUFF_SIZE);
    int left = chunk;

    // fill up
    for (int i = 0; left >= (int)sizeof (c->isr.qdata) && i < chunk && TX_SUCCESS == status; i += (int)sizeof (c->isr.qdata))
    {
        status = tx_queue_send(&c->isr.ctrl, (uint32_t *)&str[i], TX_NO_WAIT);
        // keep track of remainder
        left -= (int)sizeof (c->isr.qdata);
    }
    // any remainder?
    if (left > 0 && TX_SUCCESS == status)
    {
        // push in the last word
        c->isr.qdata = 0;
        const uint8_t *p = (const uint8_t *)&str[chunk - left];
        // push it in ("left" could be only 3, 2 or 1)
        for (int i = 0; i < left; i++)
        {
            c->isr.qdata |= (uint32_t)(*p << (i << 3));
            p++;
        }
        //
        status = tx_queue_send(&c->isr.ctrl, &c->isr.qdata, TX_NO_WAIT);
    }
}

static void x_printf_isr_heading (x_printf_context_t *c)
{
    const char header[] = "[ISR] ";

    x_printf_queue_str (c, header, sizeof (header));
}

static void x_printf_thread_create (x_printf_context_t *c)
{
    tx_semaphore_create (&c->write.start, (CHAR *) "TIMEOUT SEMAPHORE", 0);
    //
    tx_thread_create (&c->thread, (CHAR *) "X PRINTF Thread", x_printf_thread,
                      (ULONG) NULL, &c->stack, sizeof (c->stack), 1, 1, 1, TX_AUTO_START);

    //
    tx_thread_create (&c->write.watchdog, (CHAR *) "X PRINTF Thread watchdog", x_printf_thread_watchdog,
                      (ULONG) NULL, &c->write.stack, sizeof (c->write.stack), 1, 1, 1, TX_AUTO_START);
}

static void x_printf_thread (ULONG thread_input)
{
    /* Not currently using thread_input. */
    SSP_PARAMETER_NOT_USED (thread_input);

    while (1)
    {
        // if there is any ISR context log requests ... service them now
        if (0 == x_printf_isr_service ())
        {
            // nothing processed ... yield
            tx_thread_sleep (1);
        }
    }

}

static void x_printf_thread_watchdog (ULONG thread_input)
{
    x_printf_context_t *c = &x_printf_context;
    /* Not currently using thread_input. */
    SSP_PARAMETER_NOT_USED (thread_input);

    // wait for a write to start
    tx_semaphore_get(&c->write.start, TX_WAIT_FOREVER);
    // within X sec this will timeout
    tx_thread_sleep((ULONG)(c->write.timeout > 0 ? c->write.timeout : 1));
    // timeout has lapsed ... was anything written so far?
    // is there a calling thread that needs a wait to be abborted?
    if (0 == c->write.count && c->write.caller)
    {
        // nothing
        // NOTE:
        // The api->write has blocked ... the calling thread
        // is locked and any other threads that might have called x_printf
        // are suspended because of the c->lock mutex, so it's safe to do this
        // wake up the suspended thread that was trying to x_printf
        tx_thread_wait_abort(c->write.caller);
    }
}

#ifndef X_PRINTF_WAIT
#   define X_PRINTF_WAIT TX_NO_WAIT
#endif

static ssp_err_t x_printf_flush_x(x_printf_context_t *c, const char *buff, int len)
{
    ssp_err_t err;

    if (c->write.disable)
    {
        // writes have been disabled ... likely doe to USB cable not
        // connected or port not open on PC
        err = SSP_ERR_TIMEOUT;
    }
    else
    {
        // in the case of USBX CDC over USB ... the call below can block indefinitely
        // regardless of the timeout passed so the work around is to post a semaphore
        // just before we will start the write, and make another thread be the watchdog
        // of this process
        c->write.caller = tx_thread_identify ();
        tx_semaphore_put(&c->write.start);
        // this call below can block for ever at at the moment (SSP 1.0.0 Gold)
        err = c->api->write(c->ctrl, (uint8_t *)buff, (uint32_t)len, X_PRINTF_WAIT);
        //
        switch (err)
        {
            case SSP_SUCCESS:
                c->write.count++;
                break;

            case SSP_ERR_INTERNAL: // TX_WAIT_ABORTED:
            case SSP_ERR_TIMEOUT:
                // NOTE:
                // watchdog thread aborted the suspend due to timeout
                // add any additional processing here
                // we are giving up on the SF_COMM, close it
                //err = c->api->close (c->ctrl);
                // kill this
                c->ctrl = NULL;
                c->api = NULL;
                c->write.disable = true;
                c->write.caller = NULL;
                break;

            default:
                __asm__ ("nop");
                break;
        }
    }

    return err;
}

