/*
 ****************************************************************************************
 * DISCLAIMER
 ****************************************************************************************
 * This software and/or information (the "Material") is supplied by Renesas Electronics
 * Corporation. and is only intended for use with Renesas products. No other uses are authorized.
 * The Material is owned by Renesas Electronics Corporation and is protected under all applicable laws,
 * including copyright laws.
 * THIS MATERIAL IS PROVIDED "AS IS," AND RENESAS MAKES NO WARRANTIES REGARDING THIS MATERIAL,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
 * TO THE MAXIMUM EXTENT PERMITTED BY LAW, NEITHER RENESAS ELECTRONICS CORP. NOR ANY OF ITS
 * AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES FOR ANY REASON RELATED TO THIS MATERIAL, EVEN IF RENESAS OR ITS AFFILIATES
 * HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
 * Renesas reserves the right, without notice, to make changes to this Material
 * and to discontinue its availability
 *
 ****************************************************************************************
*/
/* Main app thread entry function */
#include <stdio.h>
#include "util.h"
#include "display.h"
#include "hal_data.h"
#include "led_pulse.h"
#include "pulse_adc.h"
#include "pulse_bias.h"
#include "main_thread.h"
#include "pulse_ox_thread.h"
#include "pulse_adc_ssp.h"
#include "led.h"
#include "events.h"
#include "display.h"
#include "x_print.h"
#include "pulse_adc_process.h"

//#define DO_X_PRINTF_RAW_TEST
//#define TEST_DISPLAY
//#define TEST_PULSE_R_PWM
//#define TEST_PULSE_BIAS_DAC
//#define TEST_DA0_CONNECTION
//#define TEST_SAMPLE_ADC
//#define DA0_GEBUG_LINE

#if defined (TEST_DISPLAY) || defined (TEST_PULSE_R_PWM) || defined (TEST_PULSE_BIAS_DAC) || defined (TEST_SAMPLE_ADC)
#   define TEST
#endif

static void test (void);
static void test_display (void);
static void test_pulse_leds (void);
static void test_pulse_bias_dac (void);
static void test_sample_adc (void);

enum
{
    DISPLAY_PAGE_1,
    DISPLAY_PAGE_2,
    DISPLAY_PAGE_LAST = DISPLAY_PAGE_2,
    DISPLAY_PAGE_MAX,
};

typedef struct
{
    TX_EVENT_FLAGS_GROUP *event;
    ULONG actual_flags;
    int disp;
}main_context_t;

static main_context_t context = {
    .event = &pulse_ox_app_event,
    .disp = DISPLAY_PAGE_LAST,
};

static void handle_button_up (main_context_t *c);
static void handle_show_heart_rate (main_context_t *c);
static void handle_show_oxigen_sat (main_context_t *c);
static void do_x_printf_raw_test (void);

void main_thread_entry(void)
{
    int status;
    main_context_t *c = &context;

    led_init ();

#ifdef TEST
    test ();
#endif

    pulse_bias_init ();

    // start x_print, do not use JTAG virtual console ... very SLOW
    x_print_njtag ();
    // UGLY KLUDGE
#ifdef DO_X_PRINTF_RAW_TEST
    do_x_printf_raw_test ();
#endif

    while (1)
    {
        status = tx_event_flags_get(c->event, BTN_EVENT_MASK |
                                              SHOW_ITEMS_MASK |
                                              IDLE_EVENTS_MASK, TX_OR_CLEAR,
                                    &c->actual_flags, TX_WAIT_FOREVER);

        if (status)
        {
            tx_thread_sleep (1);
        }
        else
        {
            if (BTN_UP_EVENT & c->actual_flags)
            {
                handle_button_up (c);
            }

            if (SHOW_OXIGEN_SAT_EVENT & c->actual_flags)
            {
                handle_show_oxigen_sat (c);
            }

            if (SHOW_HEART_RATE_EVENT & c->actual_flags)
            {
                handle_show_heart_rate (c);
            }

            if (IDLE_ENTER_EVENT & c->actual_flags)
            {
                pulse_ox_idle_enter ();
            }

            if (IDLE_EXIT_EVENT & c->actual_flags)
            {
                pulse_ox_idle_exit ();
            }
        }
    }
}

/**
 * some helpers below
 **/
static void do_x_printf_raw_test (void)
{
    // some crazy long wait for the USB cable to be connected and the CONSOLE to get open
    tx_thread_sleep (1000);

    // just show a hello message
    x_printff ("MAIN THREAD Started!\n\r");

    while (1)
    {
        static int i = 0;
        led_toggle (LED_1);
        x_print_raw ((uint8_t *)&i, sizeof (i));
        x_print_raw_flush ();

        i++;
        tx_thread_sleep (1);
    }
}

static void test_display (void)
{
#ifdef TEST_DISPLAY
    float value = 0;

    display_open ();

    while (1)
    {
        for (int i = 0; /*i < 999*/; i++)
        {
            display (value);

            value += 0.1;
            //
            tx_thread_sleep (4);
        }
    }
#endif
}


static void test_pulse_leds (void)
{
#ifdef TEST_PULSE_R_PWM

    pulse_r_end ();
    pulse_ir_end ();

    while (1)
    {
        //
        pulse_r_begin_kh (1, 1);
        // some wait
        tx_thread_sleep (1);
        // done
        pulse_r_end ();
        //
        pulse_ir_begin_h (1, 1);
        // some wait
        tx_thread_sleep (1);
        // done
        pulse_ir_end ();
    }
#endif
}

#ifdef TEST_PULSE_BIAS_DAC
static void test_pulse_bias_v (void)
{
    static float volt = 0;
    static float vd = 0.10f;
    // DA related vars
    float vmax = 3.30f, vmin = 0.0f;

    pulse_bias_v (volt);

    // increment
    // just bump up the value a notch
    volt += vd;
    // got to the limits ?
    if (volt > vmax || volt < vmin)
    {
        // invert
        vd *= -1;
        // bound limit
        volt = volt > vmax ? vmax : vmin;
    }
}

static void test_bulse_bias_p (void)
{
    static int percent = 0;

    pulse_bias_p (percent);

    // increment
    percent++;

    if (percent > 100)
    {
        percent = 0;
    }
}

#endif

#if defined(TEST_DA0_CONNECTION) || defined(DA0_GEBUG_LINE)

const ioport_pin_cfg_t da0 =
{
     .pin = IOPORT_PORT_00_PIN_14,
     .pin_cfg = (IOPORT_CFG_PORT_DIRECTION_OUTPUT),
};

static void pulse_bias_output_cfg (void)
{
    ioport_api_t const *api = g_ioport.p_api;

    ssp_err_t err = api->pinCfg (da0.pin, da0.pin_cfg);

    if (err != SSP_SUCCESS)
    {
        // TODO:
        // handle error
    }
}

/*static*/ void pulse_bias_toggle (void)
{
    ioport_api_t const *api = g_ioport.p_api;
    ioport_level_t value;

    ssp_err_t err = api->pinRead(da0.pin, &value);

    if (SSP_SUCCESS == err)
    {
        // toggle
        value = value == IOPORT_LEVEL_LOW ? IOPORT_LEVEL_HIGH : IOPORT_LEVEL_LOW;
        // write
        err = api->pinWrite (da0.pin, value);
    }

    if (err != SSP_SUCCESS)
    {
        // TODO: handle error condition
    }
}
#endif

static void test_pulse_bias_dac (void)
{
#ifdef TEST_PULSE_BIAS_DAC
    pulse_bias_init ();
    // this call will start the channel if it was not
    pulse_bias_v (1.65);

    while (1)
    {
        test_pulse_bias_v ();
        //test_bulse_bias_p ();
        //
        tx_thread_sleep (10);
    }
#elif defined (TEST_DA0_CONNECTION)
    pulse_bias_output_cfg ();

    while (1)
    {
        pulse_bias_toggle ();
        //
        tx_thread_sleep (1);
    }
#endif
}

static void test_sample_adc (void)
{
#ifdef TEST_SAMPLE_ADC
    int err, event, red, ired;

    //pulse_bias_output_cfg (); // just for debugging purposes

    err = pulse_adc_init ();

    if (err)
    {
        // TODO: handle error
    }

    pulse_bias_init ();

    while (1)
    {
        //test_pulse_bias_v ();
        // wait for RED led OR Infra RED led ADC value to be available
        err = pulse_adc_wait_for_event (&event, &red, &ired);
        // do something with the values given the event
        switch (event)
        {
            case PULSE_R_EVENT_FLAG:
                //pulse_bias_toggle (); just for debugging purposes
                pulse_adc_process_red (red);
                break;

            case PULSE_IR_EVENT_FLAG:
                //pulse_bias_toggle (); just for debugging purposes
                pulse_adc_process_ired (ired);
                break;

            default:
                __asm__ ("nop");
                break;
        }
    }
#endif
}

static void test (void)
{
    test_display ();
    test_pulse_leds ();
    //test_pulse_bias_dac ();
    test_sample_adc ();
}

static void handle_button_up (main_context_t *c)
{
    int ev = 0;
    c->disp = (c->disp + 1) % DISPLAY_PAGE_MAX;

    switch (c->disp)
    {
        case DISPLAY_PAGE_1:
            ev = SHOW_HEART_RATE_EVENT;
            break;

        case DISPLAY_PAGE_2:
            ev = SHOW_OXIGEN_SAT_EVENT;
            break;
    }
    //
    tx_event_flags_set (c->event, (ULONG)ev, TX_OR);
}


void pulse_ox_get (int *red, int *ired);

static void handle_show_heart_rate (main_context_t *c)
{
    uint16_t hr;

    if (DISPLAY_PAGE_1 == c->disp)
    {
        pulse_ox_get_hr (&hr);

        if (hr == 0)
        {
            display_str("---");
        }
        else
        {
            display_int(hr);
        }
    }
}

static void handle_show_oxigen_sat (main_context_t *c)
{
    uint16_t o2;

    if (DISPLAY_PAGE_2 == c->disp)
    {
        pulse_ox_get_o2 (&o2);

        if (o2 == 0)
        {
            display_str("---");
        }
        else
        {
            display_int(o2);
        }
    }
}


